@@ -22,6 +22,23 @@ type PluginCss struct {
 	Dark  string `json:"dark"`
 }
 
+const (
+	// These weights may be used by an extension to reliably place
+	// itself in relation to a particular item in the menu. The weights
+	// are negative to ensure that the default items are placed above
+	// any items with default weight.
+
+	WeightCreate = (iota - 20) * 100
+	WeightDashboard
+	WeightExplore
+	WeightProfile
+	WeightAlerting
+	WeightPlugin
+	WeightConfig
+	WeightAdmin
+	WeightHelp
+)
+
 type NavLink struct {
 	Id           string     `json:"id,omitempty"`
 	Text         string     `json:"text,omitempty"`
@@ -31,6 +48,7 @@ type NavLink struct {
 	Img          string     `json:"img,omitempty"`
 	Url          string     `json:"url,omitempty"`
 	Target       string     `json:"target,omitempty"`
+	SortWeight   int64      `json:"sortWeight,omitempty"`
 	Divider      bool       `json:"divider,omitempty"`
 	HideFromMenu bool       `json:"hideFromMenu,omitempty"`
 	HideFromTabs bool       `json:"hideFromTabs,omitempty"`
